/*                  Massachusetts Microelectronics Center           */
/*                                                                  */
/* Copyright Notice: All the modifications to the existing programs */
/*                   belong to Mass. Microelectronics Center.       */
/*                   Copyright and licensing requirements of the    */
/*                   original programs belong to the original       */
/*                   distributor.                                   */
/*                                                                  */
/* sccsid "@(#)Equations.c	40.1      meg      6/29/88"               */

#include "LookupDefs.i"

PrintEquations()
/********************************************************************
 * For each output which is a 1, print an equation to standard output 
 * which describes the output as a function of its inputs.
 */

{ int index, AlreadyFoundOne, Eliminated;
  char *name, ReadBit(), *NameFromTypeNumber();
  extern struct State *StateList;
  struct StateVectorRow * list, *MakeEquations();
  extern char * ProgramName;
  extern int NumberOfStateBits, NumberOfInputs, StateVectorSize;

  Eliminated=0;
  PrintOrderings();
  for(index=NumberOfStateBits+NumberOfInputs; index<StateVectorSize; index++)
  {/*
    * Find the name of the output variable to be processed next.
    */
    name=NameFromTypeNumber(OUTPUTS, index);
    if(name==NULL)
    { fprintf(stderr,
      "%s(PrintEquations):  Output %d not found\n", ProgramName, index);
      exit(1);
    }
    printf("%s=", name);
    list=MakeEquations(index);
    AlreadyFoundOne=FALSE;
    /*
    printf("Vector List in PrintEquations\n");
    PrintVectors(list, "foo");
    */
#ifdef COLLAPSEBUG
/* This logic has not been converted to work for meally machines.....
*/

 /*
  * A 'next state' bit is a function of the current state and inputs.
  * A regular output bit is a function only of the current state bits.
  */
    if(index<NumberOfInputs+2*NumberOfStateBits) /*A 'next state' bit*/
    {
       Eliminated=Eliminated+
         Collapse(list, 0, NumberOfInputs+NumberOfStateBits-1, 1);
    }
    else
    {  
       Eliminated=Eliminated+
         Collapse(list, NumberOfInputs, NumberOfInputs+NumberOfStateBits-1, 1);
    }
#endif

    if(list==NULL)  /*No equation for this output*/
    {  /*fprintf(stderr,
	 "%s(PrintEquations):  Output %s is never set.\n", ProgramName, name);*/
       printf("0");
    }
    else
    while(list!=NULL)
    {  if(AlreadyFoundOne)
	  printf("|\n     ");
       else
       {  AlreadyFoundOne=TRUE;
	  printf("\n     ");
       }

    /* Output bits for each vector
     */
       PrintInputs(list);
        
       list=list->next;
    }
    printf(";\n"); /*End of equations for an output*/
  }
  printf("\n\n\n");
  return(Eliminated);
} /*PrintEquations*/


struct StateVectorRow *
MakeEquations(index)
/*
 * Search every state for a vector which sets the given output to 1.
 * Generate a list of vectors, one for each description of
 * each output variable.
 *
 */
   int index;
{ 
  extern struct State *StateList;
  struct State *CurrentState;
  struct StateVectorRow *CurrentVector, *result, *temp, *CopyVector();

  result=(struct StateVectorRow *) 0;
  CurrentState=StateList;
  while(CurrentState!=NULL) /*Search every state for a vector...*/
  { CurrentVector=CurrentState->vectors;
    while(CurrentVector!=NULL)
    {  if(ReadBitByNumber(CurrentVector, index)==ONE) /*...which sets output to ONE*/
       { temp=CopyVector(CurrentVector);
	 temp->next=result;
	 result=temp;
       }
       CurrentVector=CurrentVector->next;
    }
    CurrentState=CurrentState->next;
  }
  return(result);
} /*MakeEquations*/



PrintInputs(vector)
/*********************************************************************
 * Scan all of the input bits for this vector.  If any of them are ONE
 * or ZERO then output "(<logical AND of the bits>)|"
 *********************************************************************
 */
   struct StateVectorRow *vector;
{ int BitIndex, AlreadyFoundSomething;
  char Value, *signal, *NameFromTypeNumber();
  extern int NumberOfInputs, NumberOfStateBits;
  extern char * ProgramName;

  if(vector==NULL)
  {  fprintf(stderr, "%s(PrintInputs):  Null input\n", ProgramName);
     exit(1);
  }
  AlreadyFoundSomething=FALSE;
  printf("(");
  for(BitIndex=0; BitIndex<NumberOfInputs+NumberOfStateBits; BitIndex++)
  {  signal=NameFromTypeNumber(INPUTS, BitIndex);
   /*printf("Output considered for signal %s\n", signal);*/
     if(signal==NULL)
     {  fprintf(stderr,
	"%s(PrintInputs):  Can't find input[%d]\n", ProgramName, BitIndex);
	exit(1);
     }
     Value=ReadBitByNumber(vector, BitIndex);
     if(Value==ONE)
     {  if(AlreadyFoundSomething)
	   printf("&");
	printf(" %s", signal);
	AlreadyFoundSomething=TRUE;
     }
     else
     if(Value==ZERO)
     {
	if(AlreadyFoundSomething)
	   printf("&");
	printf("!%s", signal);
	AlreadyFoundSomething=TRUE;
     }
     else
     if(Value!=DONTCARE)
     {  fprintf(stderr,
        "%s(PrintInputs):  Illegal Value '%c'\n", ProgramName, Value);
	exit(1);
     }
  }
  if(AlreadyFoundSomething==FALSE)
     printf("1");
  printf(")");
} /*PrintInputs*/


PrintOrderings()
/*****************************************************************
 * Print the INORDER and OUTORDER information required by EQNTOTT.
 *****************************************************************
 */
{ extern int StateVectorSize, NumberOfInputs, NumberOfStateBits;
  int i;
  char *name, *NameFromTypeNumber();
  extern char * ProgramName;
  printf("\nINORDER=");
  for(i=0; i<NumberOfInputs+NumberOfStateBits; i++)
  {  name=NameFromTypeNumber(INPUTS, i);
     if(name==NULL)
     { fprintf(stderr,
       "%s(PrintOrderings):  Input %d not found\n", ProgramName, i);
       exit(1);
     }
     printf("\n   %s", name);
  }

  printf(";\nOUTORDER=");
  for(i=NumberOfInputs+NumberOfStateBits; i<StateVectorSize; i++)
  {  name=NameFromTypeNumber(OUTPUTS, i);
     if(name==NULL)
     { fprintf(stderr,
       "%s(PrintOrderings):  Output %d not found\n", ProgramName, i);
       exit(1);
     }
     printf("\n   %s", name);
  }
  printf(";\n");
} /*PrintOrderings*/
