/*                  Massachusetts Microelectronics Center           */
/*                                                                  */
/* Copyright Notice: All the modifications to the existing programs */
/*                   belong to Mass. Microelectronics Center.       */
/*                   Copyright and licensing requirements of the    */
/*                   original programs belong to the original       */
/*                   distributor.                                   */
/*                                                                  */
/* sccsid "@(#)Init.c	40.1      meg      6/29/88"               */

#include <stdio.h>
#include "LookupDefs.i"
FILE *SummaryFile;
FILE *TruthTableFile;
FILE *ImplicantFile;

BitsPerState()
/**********************************************************
 * Return the number of bits in the state field of the fsm.
 **********************************************************
 */
{ int result, temp;
  extern int NumberOfStates;
  extern char * ProgramName;

  if(NumberOfStates<=1)
  { fprintf(stderr, "%s(BitsPerState):  Invalid number of states=%d.\n",
       ProgramName, NumberOfStates);
    exit(1);
  };
  temp=NumberOfStates-1;
  result=0;
  while(temp>0)
  {  result++;
     temp=temp>>1;
  }
  return(result);
} /*BitsPerState*/


Init(argc, argv)
/***********************************************************
 * This must be called before any other processing gets done.
 ***********************************************************
 */
  int argc;
  char * argv[];
{ extern int NumberOfStates, NumberOfInputs, NumberOfOutputs;
  extern int InputCounter, OutputCounter, NowAdding;
  extern int TotalEliminated, StateOffset, TableFlag, SummaryFlag;
  extern int UniqueFlag, SlangFlag, EquationFlag, ImplicantFlag, TruthTableFlag;
  extern int N2Flag, GraphFlag;
  extern char * StateBaseName, * ProgramName;
  extern struct DataItem *SymbolList;
  extern struct State *StateList;
  int loop;
  extern int GotFileName;
  FILE *freopen(), *fopen();
  extern FILE *SummaryFile;
  extern FILE *TruthTableFile;
  extern FILE *ImplicantFile;
  extern char *InputFileName;

  ProgramName=argv[0];
  GotFileName=FALSE;
  InputFileName=NULL;
  StateOffset=0;
  StateBaseName="FirstState";
  TableFlag      =  0; /*Default to no table generation*/
  SummaryFlag    =  0; /*Default to no summary info    */
  TruthTableFlag =  0; /*Default to no truth table file */
  ImplicantFlag  =  0; /*Default to no Implicant file */
  EquationFlag	 =  0;
  SlangFlag	 =  0;
  N2Flag         =  0;
  GraphFlag      =  0;
  UniqueFlag	 =  0;
  TotalEliminated=  0;
  NumberOfInputs =  0;
  NumberOfOutputs=  0;
  NumberOfStates =  0;
  InputCounter   =  0;
  OutputCounter  =  0;
  NowAdding      =ILLEGAL;
  SymbolList     =NULL;
  StateList      =NULL;
  AddItem("ANY",-2,STATE);
  for(loop=1;loop<argc;loop++)
  {  if(argv[loop][0]=='-')
     {  switch(argv[loop][1])
	{ case 't':     TableFlag=1;
			break;
	  case 's':     SummaryFlag=1;
			break;
	  case 'S':     SlangFlag=1;
			break;
	  case 'T':	TruthTableFlag=1;
			break;
	  case 'u':	UniqueFlag=1;
			break;
	  case 'i':	ImplicantFlag=1;
			break;
	  case 'e':	EquationFlag=1;
			break;
          case 'n':
          case 'N':     N2Flag=1;
                        break;
          case 'g':
          case 'G':     GraphFlag=1;
                        break;

	  case '\0':	GotFileName = TRUE;	/* Use stdin */
			break;

	  default :  fprintf(stderr,
		       "%s(Init):  Bad flag %s\n", ProgramName, argv[loop]);
		     break;
	}
     }
     else
/*
 *   The argument is the name of an input file for meg to use in place of
 *   standard input.  Use it if one has not yet been specified successfully.
 */
     if(GotFileName)
          fprintf(stderr, "%s(Init):  Command line argument '%s' ignored.\n",
		           ProgramName,                     argv[loop]);
     else
     {  if(freopen(argv[loop], "r", stdin)==NULL)
	{ fprintf(stderr, "%s(Init):  Can't open input file %s.\n",
		           ProgramName, 		    argv[loop]);
	  exit(1);
	}
	else
	GotFileName=TRUE;
	InputFileName=argv[loop];
     }
  }
  if(!GotFileName){
	printf("usage: %s [ [-option] ] inputfile\n", ProgramName);
	printf("	where: inputfile is the name of the file containing\n");
	printf("		the input. '-' indicates standard input.\n");
	printf("options:\n");
	printf("	-s	Print summary information in 'meg.summary'\n");
	printf("	-t	Print a human-readable truth table in 'meg.summary'\n");
	printf("	-T	Print a machine-readable truthtable in 'meg.tt'\n");
	printf("	-S	Print a Slang description to stdout\n");
	printf("	-i	Print kiss-style output to 'meg.imp'\n");
	printf("	-e	Print boolean equations to stdout\n");
	printf("	-n 	Print an N2 description to stdout\n");
	printf("	-N 	Print an N2 description to stdout\n");
	printf("	-g 	Print a GRAB description to stdout\n");
	printf("	-G 	Print a GRAB description to stdout\n");
	printf("\n");
	exit(1);
  }
  if(ImplicantFlag)
  { if((ImplicantFile=fopen("meg.imp", "w"))==NULL)
    { fprintf(stderr, "%s(Init): Can't ", ProgramName);
      fprintf(stderr, "open file meg.imp.\n");
      exit(1);
    }
  }
  if(TruthTableFlag)
  { if((TruthTableFile=fopen("meg.tt", "w"))==NULL)
    { fprintf(stderr, "%s(Init): Can't ", ProgramName);
      fprintf(stderr, "open file meg.tt.\n");
      exit(1);
    }
  }
  if(TableFlag||SummaryFlag)
  { if((SummaryFile=fopen("meg.summary", "w"))==NULL)
    { fprintf(stderr, "%s(Init): Can't ", ProgramName);
      fprintf(stderr, "open file meg.summary.\n");
      exit(1);
    }
    else
    { fprintf(SummaryFile, "SUMMARY INFORMATION GENERATED BY MEG FROM FILE ");
      if(GotFileName)
           fprintf(SummaryFile, "%s\n", InputFileName);
      else fprintf(SummaryFile, "<stdin>\n");
      fflush(SummaryFile);
    }
  }
} /*Init*/
