/*                  Massachusetts Microelectronics Center           */
/*                                                                  */
/* Copyright Notice: All the modifications to the existing programs */
/*                   belong to Mass. Microelectronics Center.       */
/*                   Copyright and licensing requirements of the    */
/*                   original programs belong to the original       */
/*                   distributor.                                   */
/*                                                                  */
/* sccsid "@(#)Items.c	40.1      meg      6/29/88"               */

#include "LookupDefs.i"

struct DataItem *
NewItem(name, number, type)
/********************************************************************
 * Return a pointer to a new item record representing the given data.
 * The record's "next" link is null.
 ********************************************************************
 */
   char *name;
   int number, type;
{  struct DataItem *temp;
   extern char * ProgramName;
   char *calloc();

   temp=(struct DataItem *) calloc(1, sizeof(struct DataItem));
   if(temp==0)
   {  fprintf(stderr, "%s(NewItem):  calloc failed\n", ProgramName);
      exit(1);
   };
   temp->name=name;
   temp->number=number;
   temp->type=type;
   temp->next=NULL;
   return(temp);
} /*NewItem*/


AddItem(name, number, type)
/*******************************************************************
 * Add a new symbol to the global symbol table.  The symbol table is
 * a linked list.
 *******************************************************************
 */
   char *name;
   int number, type;
{  struct DataItem *NewItem(), *temp;
   extern struct DataItem *SymbolList;
   extern char * ProgramName;
   /*printf("AddItem for %s; number %d; type %d\n", name, number, type);*/
   if((type!=INPUTS)&&(type!=OUTPUTS)&&(type!=STATE))
   {  fprintf(stderr,
	      "%s(AddItem):  Bad item type code %d\n", ProgramName, type);
      exit(1);
   }
   if(TypeFromName(name)== -1) /*Not yet defined*/
   { temp=NewItem(name, number, type);
     temp->next=SymbolList;
     SymbolList=temp;
   }
   else
   { fprintf(stderr,"%s(AddItem):  Try to redefine %s; number %d; type %d\n",
	     ProgramName, name, number, type);
     exit(1);
   }
} /*AddItem*/

ChangeItem(name, number, type)
/*******************************************************************
 * Change the name of an item in the symbol table.  The symbol table is
 * a linked list.
 *******************************************************************
 */
   char *name;
   int number, type;
{  struct DataItem *NewItem(), *temp;
   extern struct DataItem *SymbolList;
   extern char * ProgramName;
   /*printf("ChangeItem for %s; number %d; type %d\n", name, number, type);*/
   if((type!=INPUTS)&&(type!=OUTPUTS)&&(type!=STATE))
   {  fprintf(stderr,
	      "%s(ChangeItem):  Bad item type code %d\n", ProgramName, type);
      exit(1);
   }
   temp=SymbolList;
   for(;;)
   { if(temp==NULL) {
	fprintf(stderr, "%s(ChangeItem): Item not found.\n", ProgramName);
	exit(1);
     }
     else if(temp->number ==  number) {
	/*Match*/
	if (temp->type != type) {
		fprintf(stderr, "%s(ChangeItem): Type mismatch\n", ProgramName);
		exit(1);
	}
	temp->name = name;
	break;
     }
     else
     	temp=temp->next;	
   }
} /*ChangeItem*/



AddState(Name)
/**************************************************************************
 * Add another state to the SymbolList.  Increment the NumberOfStates to
 * keep track of the number to be given to the next state.
 **************************************************************************
 */
   char * Name;
{ extern int NowAdding, NumberOfStates, NumberOfInputs;
  extern char * ProgramName;

/*if(NowAdding==INPUTS)  *Making the transition to first state* 
  {  NumberOfStates=0;
     NowAdding=STATE;
  }
  else
  if(NowAdding!=STATE)
  {  fprintf(stderr, "AddState for %s while adding %d\n", Name, NowAdding);
     exit(1);
  }
 */
  if(strcmp(Name, "LOOP")==0)
  {  fprintf(stderr, "%s(AddState):  ", ProgramName);
     error("Keyword LOOP used as a state name");
     exit(1);
  }
  AddItem(Name, NumberOfStates, STATE);
  NumberOfStates++;
} /*AddState*/


AddInput(Name)
/**************************************************************************
 * Add an input signal to the SymbolList.  Increment the InputCounter to
 * keep track of the number to be given to the next input.
 **************************************************************************
 */
  char * Name;
{ extern int NowAdding, NumberOfInputs, NumberOfStateBits, InputCounter;

/*if(NowAdding==ILLEGAL) *Making transition to first input*
  { NowAdding=INPUTS;
    InputCounter=0;
  }
  else
  if(NowAdding!=INPUTS)
  {  fprintf(stderr, "AddInput for %s while adding %d\n", Name, NowAdding);
     exit(1);
  }
 */
  NumberOfInputs++;
  AddItem(Name, InputCounter, INPUTS);
  InputCounter++;
} /*AddInput*/


AddOutput(Name)
/**************************************************************************
 * Add an output state to the SymbolList.  Increment the OutputCounter to
 * keep track of the number to be given to the next output.
 **************************************************************************
 */
   char * Name;
{ extern int NowAdding, NumberOfOutputs, NumberOfStateBits, OutputCounter;
  extern char * ProgramName;

/*if(NowAdding==STATE) *Making transition to first output*
  { NowAdding=OUTPUTS;
    NumberOfStateBits=BitsPerState();
    OutputCounter=(2*NumberOfStateBits)+NumberOfInputs;
  }
  else
  if(NowAdding!=OUTPUTS)
  {  fprintf(stderr, 
     "%s(AddOutput):  Add %s while adding %d\n", ProgramName, Name, NowAdding);
     exit(1);
  }
 */
  NumberOfOutputs++;
  AddItem(Name, OutputCounter, OUTPUTS);
  OutputCounter++;
} /*AddOutput*/


TypeFromName(Name)
/********************************************************************
 * Return the type associated with the name.  Return -1 if not found.
 ********************************************************************
 */
   char *Name;
{  struct DataItem *temp;
   extern struct DataItem *SymbolList;
   temp=SymbolList;
   for(;;)
   { if(temp==NULL) return(-1); /*Not found*/
     else
     if(strcmp(temp->name, Name)==0) return(temp->type); /*Match*/
     else
     temp=temp->next;	
   }
} /*TypeFromName*/


NumberFromName(Name)
/**********************************************************************
 * Return the number associated with the name.  Return -1 if not found.
 **********************************************************************
 */
   char *Name;
{  struct DataItem *temp;
   extern struct DataItem *SymbolList;
   temp=SymbolList;
   for(;;)
   { /*printf("checking %s to see if it is %s\n", temp->name, Name);*/
     if(temp==NULL) return(-1); /*Not found*/
     else
     if(strcmp(temp->name, Name)==0) break;
     else
     temp=temp->next;	
   }
   /*printf("Number from name %s is %d\n", Name, temp->number);*/
   return(temp->number);
} /*NumberFromName*/


char *
NameFromTypeNumber(type, number)
/*
 * Return the name of the symbol with the given type and number.  If not
 * found, then return NULL.
 */
   int type, number;
{  struct DataItem *temp;
   extern struct DataItem *SymbolList;
   temp=SymbolList;
   for(;;)
   { if(temp==NULL) return(NULL); /*Not found*/
     else
     if((temp->number==number)&&(temp->type==type))
	return(temp->name); /*Match*/
     else
     temp=temp->next;
   }
} /*NameFromTypeNumber*/


MakeStateNamesNonunique()
/***********************************************************************
 ** Change the input and output state bit names to be the same.
 ** The new names are StBit<n>*, where n is the bit number.
 ***********************************************************************
 */
{ char *name, *calloc();
  extern int NumberOfStateBits, NumberOfInputs, NowAdding;
  extern char * ProgramName;
  int i;
  for(i=0;i<NumberOfStateBits;i++) {
     name=(char *) calloc(20, sizeof(char));
     sprintf(name, "StBit%01d*\0", i);
     ChangeItem(name, i+NumberOfInputs, INPUTS);
     name=(char *) calloc(20, sizeof(char));
     sprintf(name, "StBit%01d*\0", NumberOfStateBits-i-1);
     ChangeItem(name, NumberOfInputs+NumberOfStateBits+i, OUTPUTS);
  }
} /*MakeStateNamesNonunique*/

AddStateSignals()
/***********************************************************************
 * Give symbolic names to the state signals.  Add Data items of the form
 * InSt<n>*, where <n> is an integer from 0 to NumberOfStates-1.
 * The signal InSt0* is the msb of the state number.  Similarly
 * for outputs.
 ***********************************************************************
 */
{ char *name, *calloc();
  extern int NumberOfStateBits, NumberOfInputs, NowAdding;
  extern char * ProgramName;
  int i;
  /*
  if(NowAdding!=OUTPUTS)
  { fprintf(stderr, 
       "%s(AddStateSignals):  Called Before all inputs are processed.\n",
       ProgramName);
    exit(1);
  }
  */
  for(i=0;i<NumberOfStateBits;i++)
  {  name=(char *) calloc(20, sizeof(char));
     sprintf(name, "InSt%01d*\0", i);
     AddItem(name, i+NumberOfInputs, INPUTS);
     name=(char *) calloc(20, sizeof(char));
     sprintf(name, "OutSt%01d*\0", NumberOfStateBits-i-1);
     AddItem(name, NumberOfInputs+NumberOfStateBits+i, OUTPUTS);
  }
} /*AddStateSignals*/


DumpSymbols()
/*****************************
 * Print the Symbol Table List
 *****************************
 */
{ extern struct DataItem *SymbolList;
  struct DataItem *temp;
  temp=SymbolList;
  while(temp!=NULL)
  {  printf("\nSymbol %8s,      Number %4d,     Type",
	    temp->name, temp->number);
     switch(temp->type)
     {  case INPUTS:	printf("Input");    break;
	case OUTPUTS:	printf("Output");   break;
	case STATE:	printf("State");    break;
	case ILLEGAL:	printf("Illegal");  break;
	default:	printf("Error");    break;
     }
     temp=temp->next;
  }
  printf("\n");
}
