/*                  Massachusetts Microelectronics Center           */
/*                                                                  */
/* Copyright Notice: All the modifications to the existing programs */
/*                   belong to Mass. Microelectronics Center.       */
/*                   Copyright and licensing requirements of the    */
/*                   original programs belong to the original       */
/*                   distributor.                                   */
/*                                                                  */
/* sccsid "@(#)Vectors.c	40.1      meg      6/29/88"               */

#include "ParserDefs.i"
#include "LookupDefs.i"
#include "Externals.i"

SetOutputBits(Vector, signals)
/*************************************************************************
 ** Set the output bits associated with the asserted signals on this
 ** transition (Mealy machine).
 *************************************************************************
 */
   struct StateVectorRow * Vector;
   struct AssertList	 * signals;

{
	while (signals!=NULL){
		SetBit(Vector, signals->name, signals->bit);
		signals=signals->next;
	}
}


SetStateBits(VectorRow, StateName, InOrOut)
/*************************************************************************
 * Set either the current state bits or the next state bits in VectorRow->
 * to the state number of the given state.
 *
 * The format of the state vector is:
 *   Current State Bits, Inputs, Next State Bits, Outputs
 *************************************************************************
 */
   struct StateVectorRow *VectorRow;
   char * StateName;
   int InOrOut;
{  int MSBIndex, i, temp;
   extern int NumberOfStateBits, NumberOfInputs;
   extern char * ProgramName;
   unsigned mask;
   /*Decide which set of bits to set-up*/
   if(InOrOut==INPUTS)
      MSBIndex=NumberOfInputs;
   else
   if(InOrOut==OUTPUTS)
      MSBIndex=2*NumberOfStateBits+NumberOfInputs-1;
   else
   {  fprintf(stderr, 
      "%s(SetStateBits):  Bad value for InOrOutSetStateBits.\n", ProgramName);
      exit(1);
   }

   temp=NumberFromName(StateName);
   if(temp== -1)
   {  fprintf(stderr, "%s(SetStateBits):  State %s not found in lookup\n",
         ProgramName, StateName);
      abort();
   }
   if(temp == -2) {
	/* Special state ANY */
	for(i=0; i<NumberOfStateBits; i++){
      		if(InOrOut==INPUTS)
	    		VectorRow->bits[i+MSBIndex]=DONTCARE;
	 	else
	    		VectorRow->bits[MSBIndex-i]=DONTCARE;
	}
   }
   else {
   	mask= 1 << (NumberOfStateBits-1);
   	for(i=0; i<NumberOfStateBits; i++)
   	{  if( (temp&(mask>>i)) !=0 )
      		{  if(InOrOut==INPUTS)
			VectorRow->bits[i+MSBIndex]=ONE;
	 	   else
	    		VectorRow->bits[MSBIndex-i]=ONE;
      		}
      	   else
      	   {	 if(InOrOut==INPUTS)
        	    VectorRow->bits[i+MSBIndex]=ZERO;
	 	else
        	    VectorRow->bits[MSBIndex-i]=ZERO;
           }
	}
   }

   if (InOrOut == OUTPUTS)
	VectorRow->StateName = StateName;
} /*SetStateBits*/


struct StateVectorRow*
NewStateVector()
/*************************************************************
 * Return a pointer to a default-initialized state vector row.
 *************************************************************
 */
{  struct StateVectorRow *temp;
   extern int StateVectorSize;
   extern char * ProgramName;
   int i;
   char *calloc();

   temp=(struct StateVectorRow *) calloc(1, sizeof(struct StateVectorRow));
   if(temp==0)
   {  fprintf(stderr, "%s(NewStateVector):  calloc failed\n", ProgramName);
      exit(1);
   };

   temp->bits=(char *) calloc(StateVectorSize+1, sizeof(char));
   if(temp->bits==0)
   {  fprintf(stderr, "%s(NewStateVector):  calloc failed\n", ProgramName);
      exit(2);
   };

   for(i=0;i<StateVectorSize-NumberOfOutputs;i++)
      temp->bits[i]=DONTCARE;
   for(i=StateVectorSize-NumberOfOutputs;i<StateVectorSize;i++)
      temp->bits[i]=ZERO;
   temp->bits[StateVectorSize]='\0';
   temp->next=NULL;
   temp->StateName=NULL;
   return(temp);
} /*NewStateVector*/


struct StateVectorRow *
CopyVector(ToCopy)
/****************************************************
 * Return a pointer to a copy of the given row vector.
 ****************************************************
 */
   struct StateVectorRow * ToCopy;
{  struct StateVectorRow * NewStateVector(), *temp;
   extern int StateVectorSize;
   extern char *ProgramName;
   int i;

   if(ToCopy==NULL)
   {  fprintf(stderr, "%s(CopyVector):  NULL pointer passed.\n", ProgramName);
      exit(1);
   }
   temp=NewStateVector();
   for(i=0;i<StateVectorSize;i++)
      temp->bits[i]=ToCopy->bits[i];
   temp->next=ToCopy->next;
   temp->StateName=ToCopy->StateName;
   return(temp);
} /*CopyVector*/
