/*                  Massachusetts Microelectronics Center           */
/*                                                                  */
/* Copyright Notice: All the modifications to the existing programs */
/*                   belong to Mass. Microelectronics Center.       */
/*                   Copyright and licensing requirements of the    */
/*                   original programs belong to the original       */
/*                   distributor.                                   */
/*                                                                  */
/* sccsid "@(#)grabgraph.c	40.1      meg      6/29/88"               */

/*
**
**	Meg, the parent program to these routines interprets a grammar
**	for describing Mealy machine style finite state machines.  It is
**	used to generate a number of equivalent descriptions suitable for
**	automated CAD tools.  The tool these routines are aimed at is the
**	SUNGRAB directed graph editor and automated placement tool.  This
**	tool takes a simple description of the transition graph and places
**	the nodes in a nice way subject to human intervention.
**	Unfortunately the current version has very poor labeling control.
**
**	The routine PrintGraph is called by Meg after the input file has been
**	parsed into a data structure (described in ParserDefs.i) that is
**	input.  The file name of the input (and a flag indicating that the
**	input is really stdin) is also input.  PrintGraph will write to the
**	stdout a SUNGRAB file that describes the finite state machine Meg read.
**	It begins by replicating the data structure collected by the parser
**	so that it can reverse all lists without interfering with other meg
**	functions.  Lists need to be reversed because the parser stacks the
**	input structures.
**
**	The source file name is used to name the SUNGRAB graph.
**
**	Meg has a nasty habit of generating empty transition arc records at
**	the end of state cases with no explicit default transition.  Meg
**	later adjusts these to indicate sequential default arcs (ie. if no
**	default is given and no case is matched then goto the next state
**	listed in the input order).  This representation does not encode
**	this - it assumes that all transitions are explicit.  This
**	discrepancy could lead to alot of trouble!!
**
*/

#include <stdio.h>
#include "ParserDefs.i"

/* #define DBG on */

#define LSS -1		/* used for string compare results */
#define EQL 0
#define GTR 1

#define TRUE 1		/* logic values */
#define FALSE 0

#define RECTANGLE	0	/* graph node types */
#define CIRCLE		1
#define DIAMOND		2
#define OVAL		3
#define POINT		4
#define DOUBLE_BOX	5


/* this routine, PrintGraph, will eventually migrate to a file of its own.
It is intended to generate a directd graph specification that can be
postprocessed into documentation pictures.
*/

PrintGraph( List, FileSource, FileName )
struct StateRecord *List;
int FileSource;
char *FileName;
{
    struct StateRecord *state, *s, *ReverseStateList();
    struct IdList *input, *bit;
    struct IdList *Inputs, *Outputs;
    struct IdList *VectorList, *v;
    struct CaseItem *arc;
    struct AssertList *value;
    char *Prefix, *Suffix, *t, *calloc();
    int NoStates, i, FieldWidth;

    if( FileSource == FALSE || FileName == (char *) 0 ) FileName = "n2.meg";
    for( t=(char *) 0, Suffix=Prefix=FileName; *Suffix != '\0'; Suffix++ )
        if( *Suffix == '.' ) t = Suffix;
    if( t != (char *) 0 ) { *t++ = '\0'; Suffix = t; };

    printf( "*NAME*\n%s\n", Prefix );

/* sorry about the use of (type *) 0 but this is the coding convention
inherited from the originator of Meg
*/

    Inputs = (struct IdList *) 0;
    Outputs = (struct IdList *) 0;
    VectorList = (struct IdList *) 0;

/* create a copy of the parser data structure with reversed lists
*/
    s = state = ReverseStateList( List, &Inputs, &Outputs );

/* get the nodes that represent states
*/
    for( printf( "*NODES*\n" );
         state != (struct StateRecord *) 0; state = state->next )
         printf( "%s\tN:%s\t%d\n", state->name, state->name, OVAL );

/* get the nodes that represent arcs
*/
    for( state = s ; state != (struct StateRecord *) 0; state = state->next )
    {
      for( i=0, arc = state->cases;
           arc != (struct CaseItem *) 0;
           i++, arc = arc->next )
      {
        if( arc->bits == (struct IdList *) 0 && arc->StateName == (char *) 0 )
            continue;
	printf( "%s_to_%s_no_%d\tA:%s_%d\t%d\n", state->name,
		arc->StateName, i, state->name, i, RECTANGLE );
      }
    }

    printf( "*EDGES*\n" );

/* break each transition into 2 arcs with the dummy arc node in the middle
so that arcs can be labelled - see the n2 code for the label references
the labels are: A:fromState_# where # corresponds to the order that cases
appear in meg code starting at zero
*/
    for( state = s ; state != (struct StateRecord *) 0; state = state->next )
    {
      for( i=0, arc = state->cases;
           arc != (struct CaseItem *) 0;
           i++, arc = arc->next )
      {
        if( arc->bits == (struct IdList *) 0 && arc->StateName == (char *) 0 )
            continue;
        printf( "%s\t%s_to_%s_no_%d\n", state->name, state->name,
		arc->StateName, i );
        printf( "%s_to_%s_no_%d\t%s\n", state->name,
		arc->StateName, i, arc->StateName );
      }
    }
}
