/*                  Massachusetts Microelectronics Center           */
/*                                                                  */
/* Copyright Notice: All the modifications to the existing programs */
/*                   belong to Mass. Microelectronics Center.       */
/*                   Copyright and licensing requirements of the    */
/*                   original programs belong to the original       */
/*                   distributor.                                   */
/*                                                                  */
/* sccsid "@(#)main.c	40.1      meg      6/29/88"               */

#include <stdio.h>
#include "y.tab.h"
#include "LookupDefs.i"

char * ResetSignal;
struct NextState *ResetState;
extern int yychar;
extern FILE *SummaryFile;
extern FILE *TruthTableFile;
extern FILE *ImplicantFile;
/*extern int yydebug;*/
/* extern int yylineno; */
extern char yysbuf[], * yysptr;
int TotalEliminated, ImplicantFlag, TruthTableFlag, TableFlag, SummaryFlag;
int UniqueFlag, SlangFlag, EquationFlag, N2Flag, GraphFlag;
char * ProgramName;
int GotFileName;
char *InputFileName;

main(argc, argv)
   int argc;
   char * argv [];
{ extern struct StateRecord * ParserList;
  extern int SummaryFlag, TableFlag;
  int NameType;

  ResetSignal = (char*) 0;
  ResetState = (struct NextState*) 0;
  Init(argc, argv);
  yyparse();
  /*
  **   Symbolic output section
  */
  if (N2Flag)
	PrintN2(ParserList, GotFileName, InputFileName );
  if (GraphFlag)
	PrintGraph(ParserList, GotFileName, InputFileName );
#ifdef FSMDEBUG
   PrintStateList(ParserList);
#endif
   /*
   RegisterStateNames(ParserList);
   */
   FixOutputCounters();
   MakeStateList();

   AddStateSignals();
   BuildTable(ParserList);
   AddResets(ResetSignal, ResetState);

   /* 
   **	Output section.
   */
   if (! UniqueFlag)
	MakeStateNamesNonunique();
   if(SlangFlag)
	PrintSlang();
   if(EquationFlag)
   	TotalEliminated=TotalEliminated+PrintEquations();
   if(TableFlag||TruthTableFlag||ImplicantFlag)
      	PrintTable();
   if((TotalEliminated>0)&&(SummaryFlag))
   { fprintf(SummaryFile, "\n------------------------------\n%s:", ProgramName);
     fprintf(SummaryFile, "  Eliminated Vectors=%d\n\n", TotalEliminated);
   }
   CheckTable();
   if(TableFlag||SummaryFlag) fclose(SummaryFile);
   if(TruthTableFlag) fclose(TruthTableFile);

   exit(0);
} /*main*/


yyerror(s)
/*
 * Prints a message when a syntax error is detected.
 */
   char *s;
{  fprintf(stderr, "%s: unexpected token ", s);
   if(yychar>256)
   switch (yychar)
   {   case IDENTIFIER  :  fprintf(stderr, "IDENTIFIER");break;
       case INPUT 	:  fprintf(stderr, "INPUT");	break;
       case OUTPUT 	:  fprintf(stderr, "OUTPUT");	break;
       case CASE 	:  fprintf(stderr, "CASE");	break;
       case IF 	        :  fprintf(stderr, "IF");	break;
       case THEN 	:  fprintf(stderr, "THEN");	break;
       case ELSE 	:  fprintf(stderr, "ELSE");	break;
       case ARROW 	:  fprintf(stderr, "'=>'");	break;
       case DEFAULT 	:  fprintf(stderr, "DEFAULT");	break;
       case ENDCASE 	:  fprintf(stderr, "ENDCASE");	break;
       case GOTO 	:  fprintf(stderr, "GOTO");	break;
       case NOT 	:  fprintf(stderr, "'NOT'");	break;
       case COLON 	:  fprintf(stderr, "':'");	break;
       case SEMICOLON 	:  fprintf(stderr, "';'");	break;
       case LPAREN 	:  fprintf(stderr, "'('");	break;
       case RPAREN 	:  fprintf(stderr, "')'");	break;
       case ONE         :  fprintf(stderr, "'1'");      break;
       case ZERO        :  fprintf(stderr, "'0'");     break;
       case DONTCARE    :  fprintf(stderr, "'?'"); break;
       default		:  fprintf(stderr, "%d", yychar);break;
   }
   else fprintf(stderr, "(%c)", yychar);
   /* fprintf(stderr, " on line %d \n", yylineno); */
   fprintf(stderr, "\n");
   exit(1);
} /*yyerror*/

error(s)
   char * s;
{
   /* fprintf(stderr, "%s on (or before) line %d \n", s, yylineno); */
   fprintf(stderr, "%s \n", s);
} /*error*/
