
#include <stdio.h>
extern char *strcpy();

#include "mpack.h"		/* ~cad/lib/mpack/mpack.h */

/* String containing the version number of magic.  Don't change the string
 * here, nor its format.  It is updated by the Makefile in this directory. 
 */
char *MagicVersion = "Magic - Version 6.4.4";
char *MagicCompileTime = "Last updated on Thu Mar  2 11:43:43 CST 1995.";

/*
 *
 * 'MQuilt' module generator -- same as 'Quilt', except that it writes
 * Magic files instead of Caesar files.
 *
 * Copyright (C) January 1983 by Robert N. Mayo, Berkeley, CA
 *
 * Given an array of characters (each row represented by one line of
 * input text), create an array of tiles where each tile has the same
 * name as the corresponding character in the array.  Blank lines and 
 * blank characters in the input are ignored. 
 *
 * The algorithm used scans each input line left to right, placing tiles
 * next to the previously placed tile.  New input lines are placed such
 * that the first tile of the line is directly under the first tile in
 * the line above it.
 *
 * If a tile name is preceded by a single character in parenthesis, then
 * the tile of that name will be used as the spacing amount rather than
 * the size of the previous tile.
 *
 */

main(argc, argv)
int argc;
char *argv[];
{
    char tilename[2];
    char inchar;
    RECTANGLE prev_rect, start_prev_row;
    TILE out_tile, this_tile;
    int newrow, first_in_row;

    TPinitialize(argc, argv, "~cad/lib/mquilt/q"); 
    out_tile = TPcreate_tile("mquilt-out");

    (void) strcpy(tilename,"?");
    start_prev_row = origin_rect;

    newrow = 1;
    while ( (inchar = getchar()) != EOF) {
	if (inchar == '\n') {
	    newrow = 1;
	}
	else if (inchar == '(' ) {
	    tilename[0] = getchar();
	    this_tile = TPname_to_tile(tilename);
	    if (newrow) {
		prev_rect = TPdisp_tile(this_tile,
		    align(rLL(start_prev_row), tUR(this_tile)) );
		newrow = 0;
		first_in_row = 1;
	    }
	    else {
		prev_rect = TPdisp_tile(this_tile,
		    align(rLL(prev_rect), tLL(this_tile)) );
	    }
	    if (getchar() != ')' ) {
		printf("%s: Unmatched parenthesis in input file\n", argv[0]);
		exit(1);
	    }
	}
	else if (inchar != ' ') {
	    tilename[0] = inchar;
	    this_tile = TPname_to_tile(tilename);
	    if (newrow) {
		prev_rect = TPdisp_tile(this_tile,
		    align(rLL(start_prev_row), tUR(this_tile)) );
		newrow = 0;
		first_in_row = 1;
	    }
	    prev_rect = TPpaint_tile(this_tile, out_tile,
		align(rLR(prev_rect), tLL(this_tile)) );
	    if (first_in_row) {
		start_prev_row = prev_rect;
		first_in_row = 0;
	    }
	}
    }

    TPwrite_tile(out_tile, "");
    exit(0);
}
