/*                  Massachusetts Microelectronics Center           */
/*                                                                  */
/* Copyright Notice: All the modifications to the existing programs */
/*                   belong to Mass. Microelectronics Center.       */
/*                   Copyright and licensing requirements of the    */
/*                   original programs belong to the original       */
/*                   distributor.                                   */
/*                                                                  */
/* sccsid "@(#)Bits.c	40.1      peg      6/29/88"               */

/*
 * Bits.c
 *
 * Copyright (C) 1983 by Gordon T. Hamachi.
 * All rights reserved.
 */

#ifndef lint
static char sccsid[] = "@(#)Bits.c	1.3 PEG (Berkeley) 2/25/85";
#endif  not lint

#include "LookupDefs.i"
extern FILE *SummaryFile;

AssertSignal(StateName, Signal, Level)
/*********************************************************************
 * Outputs are associated with states (Moore machine model).  For each
 * row vector associated with a state, assert the given signal.
 *********************************************************************
 */
  char *StateName, *Signal, Level;
{ int temp;
  struct State * StateFromName(), *LocatedState;
  extern char * ProgramName;
  struct StateVectorRow *p;

  temp=TypeFromName(StateName);
  if(temp!=STATE)
  {  fprintf(stderr,
     "%s(AssertSignal):  %s is not a State\n", ProgramName, StateName);
     exit(1);
  }
  LocatedState=StateFromName(StateName);
  if(LocatedState->vectors==NULL)
  {  fprintf(stderr,
     "%s(AssertSignal):  No vectors in State %s\n", ProgramName, StateName);
     exit(1);
  }
  p=LocatedState->vectors;
  while(p!=NULL)
  {  SetBit(p, Signal, Level);
     p=p->next;
  }
} /*AssertSignal*/


SetBit(Vector, Signal, Level)
/*******************************************************
 * Set the given signal to the given level in the vector
 *******************************************************
 */
   struct StateVectorRow *Vector;
   char *Signal;
   char Level;

{  int SignalType, index;
   extern char * ProgramName;

   SignalType=TypeFromName(Signal);
   if((SignalType==ILLEGAL)||(SignalType==STATE))
   {  fprintf(stderr, "%s(SetBit):  Bad signal %s.\n", ProgramName, Signal);
      exit(1);
   }
   if((Level!=ONE)&&(Level!=ZERO))
   { fprintf(stderr,
     "%s(SetBit):  Bad level %c for signal %s\n", ProgramName, Level, Signal);
     exit(1);
   }
   index=NumberFromName(Signal);
   if(Vector->bits[index]!=DONTCARE)
   {  fprintf(stderr,
	      "%s(SetBit):  Resetting %s from %c to %c\n", ProgramName, Signal,
	      Vector->bits[index], Level);
      exit(1);
   }
   Vector->bits[index]=Level;
} /*SetBit*/


char
ReadBit(Vector, Signal)
/******************************************************************
 * Return the value associated with the given signal in the vector.
 ******************************************************************
 */
   struct StateVectorRow *Vector;
   char *Signal;
{  int NameType, SignalIndex;
   extern char * ProgramName;
   if(Vector==NULL)
   {  fprintf(stderr, "%s(ReadBit):  NULL vector\n", ProgramName);
      exit(1);
   }
   NameType=TypeFromName(Signal);
   if((NameType!=OUTPUTS)&&(NameType!=INPUTS))
   {  fprintf(stderr, 
      "%s(ReadBit):  Identifier %s is not a signal\n", ProgramName, Signal);
      exit(1);
   }
   SignalIndex=NumberFromName(Signal);
   return(Vector->bits[SignalIndex]);
} /*ReadBit*/


CopyOutputs(Into, From)
/*
 * Copy the output bits from FROM into INTO
 */
   struct StateVectorRow * Into, * From;
{  extern int NumberOfInputs, NumberOfOutputs, NumberOfStateBits;
   int loop, start;
   start=NumberOfInputs+2*NumberOfStateBits;
   for(loop=0;loop<NumberOfOutputs;loop++)
      Into->bits[loop+start]=From->bits[loop+start];
} /*CopyOutputs*/


PrintVector(CurrentVector, StateName)
/*
 * Print a single vector in a nice fashion.
 */
   struct StateVectorRow * CurrentVector;
   char 		 * StateName;
{  extern int StateVectorSize;
   extern char * ProgramName;
   int index;

  if(CurrentVector!=NULL)
  {  for(index=0; index<StateVectorSize;  index++)
	   fprintf(SummaryFile, "  %c", CurrentVector->bits[index]);
     fprintf(SummaryFile, "   %s\n   ", StateName); 
  }
  else
  {  fprintf(stderr, "%s(PrintVector):  null vector\n", ProgramName);
     exit(1);
  }
} /*PrintVector*/


PrintVectors(List, StateName)
/*
 * Print a list of vectors in a nicely formatted fashion
 */
   struct StateVectorRow * List;
   char 		 * StateName;
{  struct StateVectorRow *temp;

   temp=List;
   while(temp!=NULL)
   {  PrintVector(temp, StateName);
      temp=temp->next;
   }
   fprintf(SummaryFile, "\n   ");
} /*PrintVectors*/


PrintTable()
/************************************
 * Print the state table for the fsm.
 ************************************
 */
{ extern struct State *StateList;
  struct State *CurrentState;

  PrintInputNames();
  PrintOutputNames();
  PrintHeading();
  CurrentState=StateList;
  while(CurrentState!=NULL)
  {  PrintVectors(CurrentState->vectors, CurrentState->name);
     CurrentState=CurrentState->next;
  }
} /*PrintTable*/


PrintHeading()
/*****************************************
 * Print a heading across the state table.
 *****************************************
 */
{ int i;
  extern int StateVectorSize, NumberOfStateBits, NumberOfInputs,
	     NumberOfOutputs;
  fprintf(SummaryFile, "\n\nState Table\n   ");
  for(i=0; i<NumberOfInputs   ; i++) fprintf(SummaryFile, "  i");
  for(i=0; i<NumberOfStateBits; i++) fprintf(SummaryFile, "  s");
  for(i=0; i<NumberOfStateBits; i++) fprintf(SummaryFile, "  n");
  for(i=0; i<NumberOfOutputs  ; i++) fprintf(SummaryFile, "  o");
  fprintf(SummaryFile, "\n   ");

  for(i=0; i<NumberOfInputs   ; i++) fprintf(SummaryFile,"%3d",   i);
  for(i=0; i<NumberOfStateBits; i++) fprintf(SummaryFile,"%3d",   i);
  for(i=NumberOfStateBits; i>0; i--) fprintf(SummaryFile,"%3d", i-1);
  for(i=0; i<NumberOfOutputs  ; i++) fprintf(SummaryFile,"%3d",   i);
  fprintf(SummaryFile, "\n   ");
  for(i=0; i<StateVectorSize  ; i++) fprintf(SummaryFile,"---");
  fprintf(SummaryFile, "\n   ");
} /*PrintHeading*/


PrintInputNames()
/**********************************************************************
 * Print the names of input variables and the numbers assigned to them.
 **********************************************************************
 */
{ extern int NumberOfInputs, NumberOfStateBits;
  extern char * ProgramName;
  int i;
  char *name, *NameFromTypeNumber();
  fprintf(SummaryFile, "\n\nINPUTS:\n");
  for(i=0; i<NumberOfInputs; i++)
  {  name=NameFromTypeNumber(INPUTS, i);
     if(name==NULL)
     { fprintf(stderr,
       "%s(PrintInputNames):  Input %d not found\n", ProgramName, i);
       exit(1);
     }
     fprintf(SummaryFile, "   i%02d:  %s\n", i, name);
  }

  for(i=0; i<NumberOfStateBits; i++)
  {  name=NameFromTypeNumber(INPUTS, NumberOfInputs+i);
     if(name==NULL)
     { fprintf(stderr,
       "%s(PrintInputNames):  Input %d not found\n", ProgramName, i);
       exit(1);
     }
     fprintf(SummaryFile, "   s%02d:  %s", i, name);
     if(i==0)
	fprintf(SummaryFile, " (msb)\n");
     else
     if(i==NumberOfStateBits-1)
	fprintf(SummaryFile, " (lsb)\n");
     else
     fprintf(SummaryFile, "\n");
  }
} /*PrintInputNames*/


PrintOutputNames()
/***********************************************************************
 * Print the names of output variables and the numbers assigned to them.
 ***********************************************************************
 */
{ extern int NumberOfOutputs, NumberOfStateBits, NumberOfInputs;
  int i;
  char *name, *NameFromTypeNumber();
  extern char * ProgramName;
  fprintf(SummaryFile, "\n\nOUTPUTS:\n");
  for(i=0; i<NumberOfStateBits; i++)
  {  name=NameFromTypeNumber(OUTPUTS, i+NumberOfInputs+NumberOfStateBits);
     if(name==NULL)
     { fprintf(stderr,
       "%s(PrintOutputNames):  Output %d not found\n", ProgramName, i);
       exit(1);
     }
     fprintf(SummaryFile, "   n%02d:  %s", NumberOfStateBits-i-1, name);
     if(i==0)
	fprintf(SummaryFile, " (lsb)\n");
     else
     if(i==NumberOfStateBits-1)
	fprintf(SummaryFile, " (msb)\n");
     else fprintf(SummaryFile, "\n");
  }
  for(i=0; i<NumberOfOutputs; i++)
  {  name=NameFromTypeNumber(OUTPUTS, i+NumberOfInputs+2*NumberOfStateBits);
     if(name==NULL)
     { fprintf(stderr,
       "%s(PrintOutputNames):  Output %d not found\n", ProgramName, i);
       exit(1);
     }
     fprintf(SummaryFile, "   o%02d:  %s\n", i, name);
  }
} /*PrintOutputNames*/
