/*                  Massachusetts Microelectronics Center           */
/*                                                                  */
/* Copyright Notice: All the modifications to the existing programs */
/*                   belong to Mass. Microelectronics Center.       */
/*                   Copyright and licensing requirements of the    */
/*                   original programs belong to the original       */
/*                   distributor.                                   */
/*                                                                  */
/* sccsid "@(#)case.c	40.1      peg      6/29/88"               */

/*
 * case.c
 *
 * Copyright (C) 1983 by Gordon T. Hamachi.
 * All rights reserved.
 */

#ifndef lint
static char sccsid[] = "@(#)case.c	1.2 PEG (Berkeley) 2/25/85";
#endif  not lint



#include <stdio.h>
#include "LookupDefs.i"
#include "ParserDefs.i"

extern char * ProgramName;

GenerateVectorList(Count, OriginalVector)
/*
 * Generate 2**(Count) copies of OriginalVector and return them at
 * the end of OriginalVector.
 */
   int Count;
   struct StateVectorRow * OriginalVector;
{  struct StateVectorRow * step, *CopyVector();
   int loop;

   if(Count<1)
   { fprintf(stderr, "Bad count in GenerateVectorList=%d\n", Count);
     exit(1);
   }
   step=OriginalVector;
   for(loop=1; loop<(1<<Count); loop++)
   { step->next=CopyVector(OriginalVector);
     step=step->next;
   }
   step->next=(struct StateVectorRow *) 0;
} /*GenerateVectorList*/

LabelVectorList(List, BitNames, Count)
/*
 * Traverse the 2<<(Count) vectors on List.  Set the bits named on BitNames
 * to the binary values from 0 .. 2<<(Count)-1.
 */
   struct StateVectorRow * List;
   struct IdList         * BitNames;
   int 			   Count;
{  int loop;
   struct StateVectorRow * step, *half;
   if(Count==0)
      return;
   /*printf("Call to LabelVectorList with Count %d\n", Count);*/

   step=List;
   /*printf("Working on signal %s with count %d, 1..%d\n", BitNames->name, Count, (1<<(Count-1)));*/
   for(loop=0; loop<(1<<(Count-1)); loop++)
   {  /*printf("   Set %s to zero\n", BitNames->name);*/
      SetBit(step, BitNames->name, ZERO);
      step=step->next;
   }
   half=step;
   for(loop=0; loop<(1<<(Count-1)); loop++)
   {  /*printf("   Set %s to one\n", BitNames->name);*/
      SetBit(step, BitNames->name, ONE);
      step=step->next;
   }
   /*PrintVectors(List, "loop");*/
   LabelVectorList(List, BitNames->next, Count-1);
   LabelVectorList(half, BitNames->next, Count-1);
} /*LabelVectorList*/

NextIsUndefined(ThisVector)
/*
 * Return 1 if the "next state" bits for this vector are all don't-cares.
 */
   struct StateVectorRow * ThisVector;
{  int MSBIndex, i;
   extern int NumberOfStateBits, NumberOfInputs;

   MSBIndex=2*NumberOfStateBits+NumberOfInputs-1;
   for(i=0;i<NumberOfStateBits;i++)
      if(ThisVector->bits[MSBIndex-i]!=DONTCARE)
         return(0);
   return(1);
} /*NextIsUndefined*/

CaseApplies(CurrentVector, BitNames, BitValues)
/*
 * Return 1 if the current vector applies to the current state
 */
   struct StateVectorRow * CurrentVector;
   struct IdList         * BitNames, * BitValues;
{  struct IdList * nameptr, * valueptr;
   char VectorBitValue, ReadBit();
   nameptr =BitNames;
   valueptr=BitValues;
   while(nameptr!=(struct IdList *) 0)
   {  if(valueptr==(struct IdList *) 0)
      {  fprintf(stderr, "List length error in 'CaseApplies'\n");
	 exit(1);
      }
      VectorBitValue=ReadBit(CurrentVector, nameptr->name);
      if(VectorBitValue==DONTCARE)
      {/*
	* This is an error because VectorBitValue is supposed to be set
	* to either 1 or 0 in LabelVectorList.
	*/
	 fprintf(stderr, "CaseApplies: value %c in vector\n", VectorBitValue);
	 exit(1);
      }
      switch(valueptr->name[0]) /*Value in case line*/
      { case ZERO    : if(VectorBitValue!=ZERO) return(0);  break;
	case ONE     : if(VectorBitValue!=ONE)  return(0);  break;
	case DONTCARE: /*Doesn't matter what value is*/     break;
	default      :
	   fprintf(stderr, "CaseApplies: case value=%c\n", valueptr->name[0]);
	   break;
      }
      nameptr=nameptr->next;
      valueptr=valueptr->next;
   }
   return(1);
} /*CaseApplies*/

ApplyCases(ThisState)
/*
 * Traverse the list of vectors for the state.  If CASE APPLIES, then set
 * the output state bits to the given value for NEXT STATE.
 */
   struct StateRecord * ThisState;
{  struct StateVectorRow * VList;
   struct State * StateFromName(), * StateStruct;
   struct CaseItem * CList, * Default, * temp1, * temp2;
   extern int TotalEliminated, NumberOfInputs;
   int Count;

#ifdef FSMDEBUG
   printf("ApplyCases for state %s\n", ThisState->name);
#endif
   Default=ThisState->cases;
   if((Default==(struct CaseItem *) 0)||(Default->StateName==NULL))
   {  fprintf(stderr, "Default NEXT STATE missing for state %s.  Possibly a \n",
	    ThisState->name);
      fprintf(stderr,
	    "CASE with some input combinations unspecified, or an IF-THEN \n");
      fprintf(stderr, 
	    "in the last program state (next state on FALSE unspecified).\n");
      exit(1);
   }
   StateStruct=StateFromName(ThisState->name);
   VList=StateStruct->vectors;

   Count=IdListLength(ThisState->selector);
   GenerateVectorList(Count, VList);
   LabelVectorList(VList, ThisState->selector, Count);

/* Invert the case list so it is in the order of appearance.
 */
   temp1=(struct CaseItem *) 0;
   for(CList=Default->next; CList!=(struct CaseItem *) 0; CList=temp2)
   {
       temp2=CList->next;
       CList->next=temp1;
       temp1=CList;
   }
   Default->next=temp1;

/*
 * Check each vector on the state's list.  If any case line applies to
 * it, then set the next state for that vector to the given state.
 */
   while(VList!=(struct StateVectorRow *) 0)
   {  CList=Default->next;
      while(CList!=(struct CaseItem *) 0)
      {  
	 if(CaseApplies(VList, ThisState->selector, CList->bits))
	 {  
#ifdef FSMDEBUG
	    printf("   Case has next state %s\n", CList->StateName);
#endif
	    CList->caseUsed=1;
	    if(strcmp(CList->StateName, "LOOP")==0)
	       SetStateBits(VList, ThisState->name, OUTPUTS);
	    else
	       SetStateBits(VList, CList->StateName, OUTPUTS);
            /*PrintVector(VList, CList->StateName);*/
	    break;
	 }
	 CList=CList->next;
      }
      VList=VList->next;
   }
/*
 * Set all of the remaining cases to goto the default state.
 */
#ifdef FSMDEBUG
   printf("Doing defaults\n");
#endif
   VList=StateStruct->vectors;
   while(VList!=(struct StateVectorRow *) 0)
   {  if(NextIsUndefined(VList))
      {  
	 if(strcmp(Default->StateName, "LOOP")==0)
	    SetStateBits(VList, StateStruct->name, OUTPUTS);
	 else
	    SetStateBits(VList, Default->StateName, OUTPUTS);
#ifdef FSMDEBUG
	 printf("   Case has default next state %s\n", Default->StateName);
         PrintVector(VList, Default->StateName);
#endif
      }
#ifdef FSMDEBUG
      else
	 printf("   Case already defined\n");
#endif
      VList=VList->next;
   }
   CList=Default->next;
   for(Count=1; CList!=(struct CaseItem *) 0; Count++, CList=CList->next)
   {  
      if(!CList->caseUsed)
      fprintf(stderr, "%s(ApplyCases):  Case selector %d unused in state %s.\n",
		  ProgramName, Count, ThisState->name);

   }
   TotalEliminated=TotalEliminated+
      Collapse(StateStruct->vectors, 0, NumberOfInputs-1, 0);
} /*ApplyCases*/
